#!/usr/bin/env perl
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;
use strict;
use warnings;
use Test::More;
use Test::Exception;

{
    package Foo::Meta::Constructor1;
    use Mouse::Role;
}

{
    package Foo::Meta::Constructor2;
    use Mouse::Role;
}

{
    package Foo;
    use Mouse;
    Mouse::Util::MetaRole::apply_metaroles(
        for             => __PACKAGE__,
        class_metaroles => { constructor => ['Foo::Meta::Constructor1'] },
    );
}

{
    package Foo::Sub;
    use Mouse;
    Mouse::Util::MetaRole::apply_metaroles(
        for             => __PACKAGE__,
        class_metaroles => { constructor => ['Foo::Meta::Constructor2'] },
    );
    extends 'Foo';
}

{
    package Foo::Sub::Sub;
    use Mouse;
    Mouse::Util::MetaRole::apply_metaroles(
        for             => __PACKAGE__,
        class_metaroles => { constructor => ['Foo::Meta::Constructor2'] },
    );
    ::lives_ok { extends 'Foo::Sub' } "doesn't try to fix if nothing is needed";
}

done_testing;
